package org.zeroturnaround.javarebel.integration.servlet.weblogic;

import java.io.File;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.Logger;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.integration.servlet.util.IoUtil;
import org.zeroturnaround.javarebel.integration.servlet.util.ServletUtil;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>weblogic.servlet.internal.ServletStubImpl<code>.
 * <p>
 * Used for Weblogic 9.x and 10.x JSP updates.
 * 
 * @author Rein Raudjärv
 */
public class ServletStubImplCBP extends JavassistClassBytecodeProcessor {
  
  private static final Logger log = LoggerFactory.getInstance();

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    try {
      CtMethod m = ctClass.getDeclaredMethod("execute", new CtClass[] {
          cp.get("javax.servlet.ServletRequest"),
          cp.get("javax.servlet.ServletResponse"),
          cp.get("weblogic.servlet.internal.FilterChainImpl"),
      });
      m.insertBefore("{" +
      		"  " + ServletStubImplCBP.class.getName() + ".execute(" +
      		"    $0.context, $0.context.getRootTempDir(), $1, $2);" +
      		"}");
    }
    catch (NotFoundException e) {
      /*
       * Weblogic 8 has no such method but works correctly
       * because of WebAppServletContext.getResourceAsSource() method.
       */
    }
  }

  public static void execute(ServletContext ctx, File rootTempDir, ServletRequest req, ServletResponse resp) {
    try {
      String path = ServletUtil.getUriFromRequest((HttpServletRequest) req);
      
      String lc = path.toLowerCase();
      if (lc.endsWith(".jsp") || lc.endsWith(".jspx")) {
        File warTempDir = new File(rootTempDir, "war");
        if (!warTempDir.exists())
          return;
        
        File tempFile = new File(warTempDir, path);
        
        String realPath = ctx.getRealPath(path);
        if (realPath == null)
          return;
        
        File realFile = new File(realPath);
        long realDate = realFile.lastModified();
        
        if (realDate == 0) {
          // Removed
          if (tempFile.exists()) {
            if (tempFile.delete())
              if (log.isEnabled())
                log.log("JSP file '" + path + "' removed.");
              else if (log.isEnabled())
                log.log("Could not remove JSP file '" + path + "'.");
          }
        }
        else {
          // Maybe updated
          if (realDate > tempFile.lastModified()) {
            IoUtil.copy(realFile, tempFile);
            if (log.isEnabled())
              log.log("JSP file '" + path + "' updated.");
          }
        }
      }
    } catch (Throwable e) {
      log.log("BEA Webloigc JSP change could not be handled");
      log.error(e);
    }
  }
  
}
