package org.zeroturnaround.javarebel.integration.servlet.websphere;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.RebelServletContext;
import org.zeroturnaround.javarebel.integration.generic.BaseServletContextImplClassBytecodeProcessor;
import org.zeroturnaround.javarebel.integration.servlet.JavaRebelResourceServlet;

/**
 * Transforms <code>com.ibm.ws.webcontainer.webapp.WebApp</code>
 * by registering Websphere Servlet Context instance with JavaRebel.
 * <p>
 * Also {@link JavaRebelResourceServlet} is registered as the default Servlet.
 * 
 * @author Rein Raudjärv
 */
public class WebappCBP extends BaseServletContextImplClassBytecodeProcessor {
  
  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    super.process(cp, cl, ctClass);
    
    cp.importPackage("org.zeroturnaround.javarebel");

    // Register Servlet Context
    try {
      CtMethod m = ctClass.getDeclaredMethod("commonInitializationFinish");
      m.insertBefore(
          "ServletIntegrationFactory.getInstance().registerServletContext((" + RebelServletContext.class.getName() + ") $0);");
    } catch (NotFoundException e) {
    }
    
    try {
      registerRebelServlet(cp, cl, ctClass);
    } catch (CannotCompileException e) {
      LoggerFactory.getInstance().errorEcho(e);
    }
  }

  private void registerRebelServlet(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    final boolean verbose = LoggerFactory.getInstance().isEnabled();
    
    final String shouldDefineRebelServlet = "__shouldDefineRebelServlet";
    
    cp.importPackage("java.util");

    ctClass.addMethod(CtNewMethod.make(
        "private boolean " + shouldDefineRebelServlet + "() {" +
        "  ServletIntegration integration = ServletIntegrationFactory.getInstance();" +
        "  if (!integration.hasReplacedResources((RebelServletContext) $0))" +
        "    return false;" + // no rebel.xml found

        "  Collection mappings = config.getServletMappings().values();" +
        "  Iterator it = mappings.iterator();" +
        "  while (it.hasNext()) {" +
        "    Collection list = (Collection) it.next();" +
        "    Iterator it2 = list.iterator();" +
        "    while (it2.hasNext()) {" +
        "      com.ibm.ws.webcontainer.servlet.ServletMapping mapping = (com.ibm.ws.webcontainer.servlet.ServletMapping) it2.next();" +
        "      if (\"/\".equals(mapping.getUrlPattern()))" +
        "        return false;" + // default servlet already defined
        "    }" +
        "  }" +
        "  return true;" + // rebel.xml found and no default servlet defined
        "}",
        ctClass));
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("initializeTargetMappings");
      m.insertBefore(
          "{" +
          "  try {" +
          (verbose ? "    LoggerFactory.getInstance().log(\"Servlet mappings: \" + config.getServletMappings());" : "") +
          "    boolean shouldDefine = " + shouldDefineRebelServlet + "();" +
          (verbose ? "    LoggerFactory.getInstance().log(\"Rebel Servlet should be added: \" + shouldDefine);" : "") +

          "    if (shouldDefine) {" +
          "      String name = \"" + JavaRebelResourceServlet.class.getName() + "\";" +
          
          "      com.ibm.ws.wswebcontainer.servlet.ServletConfig info = new com.ibm.ws.wswebcontainer.servlet.ServletConfig(name);" +
          "      info.setServletName(name);" +
          "      info.setName(name);" +
          "      info.setIsJsp(false);" +
          "      info.setClassName(name);" +
          "      info.setFileName(null);" +
          "      info.setDisplayName(name);" +
          "      info.setStartUpWeight(new Integer(1));" +
          
          "      config.addServletInfo(name, info);" +
          "      config.addServletMapping(name, \"/\");" +
          "    }" +
          "  } catch (Throwable e) {" +
          "    LoggerFactory.getInstance().log(\"Could not register JavaRebelResourceServlet:\");" +
          "    LoggerFactory.getInstance().error(e);" +
          "  }" +
          "}");
    } catch (NotFoundException e) {
    }
  }
}
